package net.wanji.utc.hisense.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.vo.TempSchemeSendVO;
import net.wanji.utc.hisense.service.ControlCommandService;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author duanruiming
 * @date 2023/05/08 10:45
 */
@Api(value = "信号指令服务", tags = "信号指令服务")
@RequestMapping("/controlCommand")
@RestController
@RequiredArgsConstructor
public class  ControlCommandController {

    private final ControlCommandService controlCommandService;

    /**
     * 信号机方案下发
     *
     * @param schemeSendVO 方案下发VO
     * @return
     */
    @AspectLog(description = "信号机方案下发-基础方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public JsonViewObject schemeSend(@RequestBody @Validated SchemeSendVO schemeSendVO) throws Exception {
        return controlCommandService.schemeSend(schemeSendVO);
    }

    /**
     * 下发信号机计划
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    @AspectLog(description = "计划下发-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public JsonViewObject planSend(@RequestBody @Validated PlanSendVO planSendVO) throws Exception {
        return controlCommandService.planSend(planSendVO);
    }

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "时间表下发-时间表信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public JsonViewObject scheduleSend(@RequestBody @Validated ScheduleSendVO scheduleSendVO) throws Exception {
        return controlCommandService.scheduleSend(scheduleSendVO);
    }

    /**
     * 相位配时下发-（下发相位参数）
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    @AspectLog(description = "相位配时下发-（下发相位参数）", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public JsonViewObject phaseTimingSend(@RequestBody @Validated PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        return controlCommandService.phaseTimingSend(phaseTimingSendVO);
    }

    /**
     * 全红控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "全红控制-路口全红控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public JsonViewObject allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandService.setSignalControl(commandVO);
    }

    /**
     * 黄闪控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "黄闪控制-路口黄闪控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public JsonViewObject yellowLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandService.setSignalControl(commandVO);
    }

    /**
     * 关灯控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "关灯控制-路口关灯控制/开灯", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public JsonViewObject closeLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandService.setSignalControl(commandVO);
    }

    /**
     * 步进控制
     *
     * @param crossCode 信号id
     * @param command   命令
     * @param stepNum   一步num
     * @return
     */
    @AspectLog(description = "步进控制-步进控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossCode", value = "13位路口编号", paramType = "query", required = true, dataType = "String"),
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public JsonViewObject stepControl(@RequestParam String crossCode,
                                      @RequestParam Integer command,
                                      @RequestParam int stepNum) throws Exception {
        return controlCommandService.stepControl(crossCode, command, stepNum);
    }

    /**
     * 恢复时间表
     *
     * @param crossCode
     * @return
     */
    @AspectLog(description = "恢复时间表-恢复路口时间表执行(恢复固定配时)", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行(恢复固定配时)", notes = "恢复时间表-恢复路口时间表执行(恢复固定配时)")
    @PostMapping("/recoverSchedule")
    public JsonViewObject recoverSchedule(@RequestParam @Validated String crossCode) throws Exception {
        return controlCommandService.recoverSchedule(crossCode);
    }

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "锁定控制-相位锁定/解锁", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public JsonViewObject lockControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandService.lockControl(commandVO);
    }

    /**
     * 相位差下发
     */
    @AspectLog(description = "相位差下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public JsonViewObject phaseDiffSend(String crossCode) {
        return controlCommandService.phaseDiffSend();
    }

    /**
     * 临时方案下发
     */
    @AspectLog(description = "临时方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "临时方案下发", notes = "临时方案下发")
    @PostMapping("/tempSchemeDetail")
    public JsonViewObject tempSchemeDetail(@RequestBody @Validated TempSchemeSendVO tempSchemeSendVO)  throws Exception{
        return controlCommandService.tempSchemeDetail(tempSchemeSendVO);
    }

    /**
     * 延长相位
     */
    @AspectLog(description = "延长相位", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "延长相位", notes = "延长相位")
    @PostMapping("/extendPhase")
    public JsonViewObject extendPhase(@RequestBody @Validated ExtendPhaseDTO extendPhaseDTO)  throws Exception{
        return controlCommandService.extendPhase(extendPhaseDTO);
    }

    /**
     * 步进相位
     */
    @AspectLog(description = "步进相位-神思定制化(废弃)", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "步进相位", notes = "步进相位")
    @PostMapping("/stepingPhase")
    public JsonViewObject stepingPhase(@RequestBody @Validated StepingPhaseDTO stepingPhaseDTO)  throws Exception{
        return controlCommandService.stepingPhase(stepingPhaseDTO);
    }

}
